#pragma once

// Defines the current graphics viewport.
extern struct vdc_rectangle_t vdc_graphics_viewport;

// Defines the plotting mode.
enum vdc_graphics_plot_mode_t {
	// Plot the colour specified.
	VDC_GRAPHICS_PLOT_PLOT = 0,
	// OR the specified colour with the colour that is already there.
	VDC_GRAPHICS_PLOT_OR = 1,
	// AND the specified colour with the colour that is already there.
	VDC_GRAPHICS_PLOT_AND = 2,
	// Exclusive-OR the specified colour with the colour that is already there.
	VDC_GRAPHICS_PLOT_EOR = 3,
	// Invert the colour that is already there.
	VDC_GRAPHICS_PLOT_INVERT = 4,
};

// Stores the current graphics colour.
extern unsigned int vdc_graphics_colour;

// Stores the current graphics plotting mode.
extern enum vdc_graphics_plot_mode_t vdc_graphics_plot_mode;

// Resets the graphics viewport, clears the screen to black, sets the colour to white and the plotting mode to "plot".
void vdc_graphics_reset(void);

// Set the current colour and plotting mode.
void vdc_graphics_set_colour_and_plot_mode(unsigned int colour, enum vdc_graphics_plot_mode_t plot_mode);

// Set the current colour and sets the plotting mode to plot.
void vdc_graphics_set_colour(unsigned int colour);

// Clears the graphics viewport.
void vdc_graphics_clear(void);

// Sets a pixel.
void vdc_graphics_set_pixel(int x, int y);

// Clears a pixel.
void vdc_graphics_clear_pixel(int x, int y);

// Inverts a pixel.
void vdc_graphics_invert_pixel(int x, int y);

// Draws a pixel using the current plotting mode and colour.
extern void (*vdc_graphics_draw_pixel)(int, int);

// Draws a span using the current plotting mode and colour.
void vdc_graphics_draw_span(int left, int right, int y);

// Draws a line.
// Adapted from http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
void vdc_graphics_draw_line(struct vdc_point_2d_t a, struct vdc_point_2d_t b);

// Draws a circle.
// Adapted from http://en.wikipedia.org/wiki/Midpoint_circle_algorithm
void vdc_graphics_draw_circle(struct vdc_point_2d_t origin, unsigned int radius);

// Fills a circle.
void vdc_graphics_fill_circle(struct vdc_point_2d_t origin, unsigned int radius);

// Draws an ellipse.
void vdc_graphics_draw_ellipse(struct vdc_point_2d_t origin, unsigned int radius_x, unsigned int radius_y);

// Fills an ellipse.
void vdc_graphics_fill_ellipse(struct vdc_point_2d_t origin, unsigned int radius_x, unsigned int radius_y);

// Fills a triangle.
void vdc_graphics_fill_triangle(struct vdc_point_2d_t a, struct vdc_point_2d_t b, struct vdc_point_2d_t c);

// Draws a rectangle.
void vdc_graphics_draw_rectangle(struct vdc_rectangle_t rect);

// Fills a rectangle.
void vdc_graphics_fill_rectangle(struct vdc_rectangle_t rect);
